/*
 * Created on Tue Mar 07 2023
 *
 * Copyright (C) 2023 PixelApps - All Rights Reserved
 * The code in this file is Intellectutal Property of PixelApps
 * and can't be edited, redistributed or used without a valid license of it's product (Welltested)[https://welltested.ai]
 */
import 'package:analyzer/dart/ast/ast.dart';
import 'package:analyzer/dart/ast/visitor.dart';
import 'package:analyzer/dart/element/element.dart';
import 'package:welltested/src/executors/path_executor.dart';
import 'package:welltested/src/models/variable.dart';

class FunctionBodyVisitor extends RecursiveAstVisitor {
  FunctionBodyVisitor({required this.pathExecutor});

  final PathExecutor pathExecutor;

  final List<ClassElement> models = [];
  final List<Element> importModels = [];
  final List<List<SimpleIdentifier>> callFlows = [];
  final List<Variable> updateVariableNameNType = [];

  void addToClassElements(ClassElement model) {
    if (!models.map((e) => e.name).contains(model.name)) {
      models.add(model);
    }
  }

  List<SimpleIdentifier> getListOfPropAccess(PropertyAccess node) {
    final List<SimpleIdentifier> flows = [];
    for (final entity in node.childEntities) {
      if (entity is PrefixedIdentifier) {
        flows.addAll(extractPrefixedIdentifier(entity));
      }
      if (entity is SimpleIdentifier) {
        flows.add(entity);
      }
      if (entity is PropertyAccess) {
        flows.addAll(getListOfPropAccess(entity));
      }
    }

    return flows;
  }

  List<SimpleIdentifier> extractPrefixedIdentifier(
      PrefixedIdentifier prefixedIdentifier) {
    final List<SimpleIdentifier> flows = [];
    for (final entity in prefixedIdentifier.childEntities) {
      if (entity is PrefixedIdentifier) {
        flows.addAll(extractPrefixedIdentifier(prefixedIdentifier));
      }
      if (entity is SimpleIdentifier) {
        flows.add(entity);
      }
      if (entity is PropertyAccess) {
        flows.addAll(getListOfPropAccess(entity));
      }
    }
    return flows;
  }

  @override
  visitPropertyAccess(PropertyAccess node) {
    final flow = getListOfPropAccess(node);
    callFlows.add(flow);
  }

  @override
  visitVariableDeclarationList(VariableDeclarationList node) {
    for (final entity in node.childEntities) {
      if (entity is VariableDeclaration) {
        updateVariableNameNType.add(Variable(
            name: entity.declaredElement!.name,
            type: '${entity.declaredElement!.type}'));
      }
    }

    return super.visitVariableDeclarationList(node);
  }

  @override
  visitSimpleIdentifier(SimpleIdentifier node) {
    final staticElement = node.staticElement;

    if (staticElement is ClassElement || staticElement is EnumElement) {
      importModels.add(staticElement!);
    }
    if (staticElement is ClassElement &&
        !staticElement.library.isInSdk &&
        !staticElement.isAbstract &&
        !staticElement.isPrivate &&
        !staticElement.isSealed &&
        staticElement.librarySource.uri.path
            .contains(pathExecutor.packageName)) {
      addToClassElements(staticElement);
    }
    return super.visitSimpleIdentifier(node);
  }
}
